from __future__ import division

from traits.api import HasTraits, File, Instance, Enum, Array, Property, Range, \
     Button, on_trait_change
from traitsui.api import View, Item, HGroup, VGroup, Group
from chaco.api import ArrayPlotData, Plot
from enable.api import ComponentEditor
import numpy as np 

# Scikit-image imports
import skimage.io as skio
import skimage.color as skcolor
from skimage import img_as_ubyte #Change image data from float to int

#http://www.rapidtables.com/convert/color/rgb-to-cmyk.htm
def rgb2cmky(rgb_array):
    """ Convert rgb colors to cmky"""
    R = rgb_array[:,:,0]
    G = rgb_array[:,:,1]
    B = rgb_array[:,:,2]
    
    K = 1.0 - rgb_array.max() #(single-value right?)
    C = (1.0 - R - K) / (1.0 - K)
    M = (1.0 - G - K) / (1.0 - K)
    Y = (1.0 - B - K) / (1.0 - K)
    
    # fill a new array
    out = np.empty(rgb_array.shape)
    out[:,:,0] = C
    out[:,:,1] = M
    out[:,:,2] = Y
    return out
    

class Image2Image(HasTraits):
    """ Viewer Class to compare two images.  Each image is an instance of chaco
    ImagePlot.
    """
    
    # Traits
    path = File('/home/glue/Desktop/monolisa/mona_prado_lowres.jpg') #Default to empty, will be set in set_defaults() below
    colorspace = Enum('rgb', 'hsv')

    # Image stored in numpy arrays via scikit-image
    # image_data trait monitor it for changes and triggers events
    image = Array    
    image_modified = Array

    # Actual plot objects
    plot1 = Instance(Plot)
    plot2 = Instance(Plot)
    
    # Plot data is associated with a plot, so changes in data will trigger plot redraws
    plot1_data = ArrayPlotData
    plot2_data = ArrayPlotData
    
    # Color traits (these are actually scale factors, so 1 means full hue, 0 means no hue)
    hue = Range(0.0, 1.0, value=1.0)
    sat = Range(0.0, 1.0, value=1.0)
    value = Range(0.0, 1.0, value=1.0)
    
    red = Range(0.0, 1.0, value=1.0)
    green = Range(0.0, 1.0, value=1.0)
    blue = Range(0.0, 1.0, value=1.0)    

    reset = Button    
    
    # Define layout of User Interface
    traits_view = View(
             Item('path', label='Image Path'),
             Item('reset'),
             Group(
                 VGroup(
                      Item('hue'),
                      Item('sat', label='Saturation'),
                      Item('value'),
                  label='HSV Colors'
                     ),
                  VGroup(            
                      Item('red'),
                      Item('green'),
                      Item('blue'),
                  label='RGB Colors'),
             layout='tabbed'),
             # Align plots horizontally and use special "componenteditor" to 
             # display them correctly
             HGroup(
                 Item('plot1', show_label=False, editor=ComponentEditor() ),
                 Item('plot2', show_label=False, editor=ComponentEditor() )
             ),
          width=700,
          height=500,
          resizable=True, 
          title="GWU Image Comparison Tool",
       )
    
    def __init__(self, *args, **kwargs):
        # Super intializes all traits from HasTraits class
        super(Image2Image, self).__init__(*args, **kwargs)
        
        # Rotate self.image because scikit-image acutally displays upside down!

        def flip_180(image):
            """ Roate an image 180 degrees than flip horizontally.  Most image viewers
            do this when output image (ie matplotlib.imshow() ) but chaco doesn't"""
            rotated = np.rot90(np.rot90(image))
            return np.fliplr(rotated)
        
        self.image, self.image_modified = flip_180(self.image), flip_180(self.image_modified)
                
        # Create two plots, and set plotdata traits
        self.plot1_data = ArrayPlotData(imagedata = self.image)
        self.plot1 = Plot(self.plot1_data)
        self.plot1.img_plot("imagedata")
        
        self.plot2_data = ArrayPlotData(imagedata = self.image_modified)
        self.plot2 = Plot(self.plot2_data)
        self.plot2.img_plot("imagedata")

    #http://scikit-image.org/docs/dev/auto_examples/plot_tinting_grayscale_images.html    
    @on_trait_change('hue, sat, value, red, green, blue')
    def colorize(self):
        """ Add color of the given hue to an RGB image.
    
        By default, set the saturation to 1 so that the colors pop!
        """
     
        rgb_multiplier = [self.red, self.green, self.blue]
        hsv_multiplier = [self.hue, self.sat, self.value]
        
        # Scale by RGB values
        image_mod = self.image * rgb_multiplier       
        
        # Transform to HSV space
        image_mod = skcolor.rgb2hsv(image_mod) 
        
        # Scale by HSV values, scale, convert back to RGB
        image_mod = image_mod * hsv_multiplier
        image_mod  = skcolor.hsv2rgb(image_mod)
       
#        print rgb2cmky(image_mod)
        
        # Conversions changed data from ints (0,1...255)
        # to floats (0, 0.1 ... 1.0)
        # This converts back to ints and scale up to 255
        image_mod = img_as_ubyte(image_mod / 255.0 )        
        

        # Trigger update 
        self.image_modified = image_mod
                    
       
    def _reset_fired(self):
        for trait in ['hue', 'sat', 'val', 'red', 'green', 'blue']:
            setattr(self, trait, 1.0)        
    
    # Default values for image and image_modified (set to same default image)
    def _image_default(self):
        return skio.imread(self.path)
    
    def _image_modified_default(self):
        return skio.imread(self.path)    
    
    def _image_changed(self):
        """ If primary image changes (ie user selects new image...)"""
        # Should inspect that isn't gray image
        self.plot1_data.set_data('imagedata', self.image)
        self.image_modified = self.image
        
    def _image_modified_changed(self):
        self.plot2_data.set_data('imagedata', self.image_modified )
    
        
    # Event Handlers (must conform to _x_changed() syntax 
    # --------------
    def _path_changed(self):
        """ Change file path, reset both images, reset all slider defaults."""
        self.image = skio.imread(self.path)      
        
        

if __name__ == '__main__':
    # Create obj
    img2img = Image2Image()

    # Configure traits
    img2img.configure_traits()